/* This C# example is provided in order to help the customer in writing
 * a program that communicates with XCD contoller through RS232 channel.
 * 
 * The example is not indended to be a practical application; neither it 
 * is fully tested.
 * 
 * For details of host communication, read XCD Controller User Manual.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace XCD_Communication {
    public partial class Form1 : Form {

        Communication Comm = new Communication();

        string[] VarNames = {       // For example
            "VEL",
            "FPOS",
            "V1"
        };

        short[] VarIds = {
            1,
            9,
            1001
        };

        public Form1() {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e) {
            try {
                Comm.PortName = "COM1";        // Change to correct port name (COM2, COM3, etc.)
                Comm.OpenPort();
            }
            catch {
                MessageBox.Show("Cannot open " + Comm.PortName);                
                Application.Exit();
            }
            comboBoxVarName.Items.AddRange(VarNames);
        }

        private void ButtonCommand_Click(System.Object sender, System.EventArgs e) {
            byte Code;
            double[] Par;
            try {
                Code = Convert.ToByte(TextBoxCCode.Text);
            }
            catch {
                MessageBox.Show("Invalid command");
                return;
            }
            try {
                string[] Lex = TextBoxCParameter.Text.Split(", \t".ToCharArray(), 10, StringSplitOptions.RemoveEmptyEntries);
                Par = Array.ConvertAll<string, double>(Lex, Convert.ToDouble);
            }
            catch {
                MessageBox.Show("Invalid parameter(s)");
                return;
            }
            try {
                Comm.ExecuteCommand(Code, Par, true);
            }
            catch {
                MessageBox.Show("Command execution failed");
                return;
            }
        }

        private void TextBoxCCode_KeyPress(object sender, KeyPressEventArgs e) {
            e.Handled = !(char.IsDigit(e.KeyChar) | char.IsControl(e.KeyChar));
            if (e.KeyChar == '\r') {
                ButtonCommand_Click(sender, new EventArgs());
            }
        }

        private void TextBoxCParameter_KeyPress(object sender, KeyPressEventArgs e) {
            e.Handled = !(char.IsDigit(e.KeyChar) | e.KeyChar == '+' | e.KeyChar == '-' | e.KeyChar == '.' | e.KeyChar == ' ' | e.KeyChar == ',' | char.IsControl(e.KeyChar));
            if (e.KeyChar == '\r') {
                ButtonCommand_Click(sender, new EventArgs());
            }
        }

        private void buttonReadVar_Click(object sender, EventArgs e) {
            UpdateVarValue();
        }

        private void UpdateVarValue() {
            int Var = comboBoxVarName.SelectedIndex;
            if (Var >= 0) {
                Var = VarIds[Var];
                if (Var >= 0) {
                    try {
                        float Val = Comm.ReadVar((short)Var);
                        textBoxVarValue.Text = Val.ToString();
                    }
                    catch (Exception ex) {
                        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        textBoxVarValue.Text = "";
                    }
                } else {
                    textBoxVarValue.Text = "";
                }
            } else {
                textBoxVarValue.Text = "";
            }
        }

        private void comboBoxVarName_SelectedIndexChanged(object sender, EventArgs e) {
            UpdateVarValue();
        }

        private void textBoxVarValue_KeyPress(object sender, KeyPressEventArgs e) {
            e.Handled = !(char.IsDigit(e.KeyChar) | e.KeyChar == '+' | e.KeyChar == '-' | e.KeyChar == '.' | char.IsControl(e.KeyChar));
            if (e.KeyChar == '\r') {
                int Var = comboBoxVarName.SelectedIndex;
                if (Var >= 0) {
                    Var = VarIds[Var];
                    if (Var >= 0) {
                        try {
                            float Val = Convert.ToSingle(textBoxVarValue.Text);
                            Comm.WriteVar((short)Var, Val);
                        }
                        catch (Exception ex) {
                            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                    }
                }
            }
        }
    }
}